
/* dictionary.cc */

#include "dictionary.h"

#ifdef USE_OWN_DICT
   #include "own-dictionary.cc"

#elif defined (USE_COMPLETE_DICT)
   #include "complete-dictionary.cc"

#else
   #include <map>
   #include MAP_INCL

   #ifdef USE_STR_HASH
      #include "strhash.h"
   #endif
#endif

OPEN_NAMESPACE

#if ! defined (USE_OWN_DICT) && ! defined (USE_COMPLETE_DICT)

/******************************** STD MAP *********************************/

/* DictLongToLong */

typedef MAP_TYPE <long, long> MLL; // use hash map if available

bool DictLongToLong::contains (Key inx)
{
   return (*(MLL*)m).find (inx) != (*(MLL*)m).end ();
}

DictLongToLong::Data DictLongToLong::get (Key inx)
{
   MLL::iterator it = (*(MLL*)m).find (inx);
   if (it == (*(MLL*)m).end ())
      return 0;
   else
      return it->second;
}

void DictLongToLong::store (Key inx, Data value)
{
   // (*(MLL*)m) [inx] = value;
   (*(MLL*)m).insert (make_pair (inx, value));
}

void DictLongToLong::clear ()
{
   (*(MLL*)m).clear ();
}

DictLongToLong::DictLongToLong ()
{
   m = (void *) new MLL;
}

DictLongToLong::~DictLongToLong ()
{
   delete (MLL*) m;
}

/* DictLongToString */

typedef MAP_TYPE <long, string> MLS; // use hash map if available

bool DictLongToString::contains (Key inx)
{
   return (*(MLS*)m).find (inx) != (*(MLS*)m).end ();
}

DictLongToString::Data DictLongToString::get (Key inx)
{
   MLS::iterator it = (*(MLS*)m).find (inx);
   if (it == (*(MLS*)m).end ())
      return "";
   else
      return it->second;
}

void DictLongToString::store (Key inx, Data value)
{
   // (*(MLS*)m) [inx] = value;
   (*(MLS*)m).insert (make_pair (inx, value));
}

void DictLongToString::clear ()
{
   (*(MLS*)m).clear ();
}

DictLongToString::DictLongToString ()
{
   m = (void *) new MLS;
}

DictLongToString::~DictLongToString ()
{
   delete (MLS*) m;
}

/* DictStringToLong */

#ifdef USE_STR_HASH
   typedef MAP_TYPE <string, long, StrHash> MSL;
#else
   typedef MAP_TYPE <string, long> MSL;
#endif

bool DictStringToLong::contains (Key inx)
{
   return (*(MSL*)m).find (inx) != (*(MSL*)m).end ();
}

DictStringToLong::Data DictStringToLong::get (Key inx)
{
   MSL::iterator it = (*(MSL*)m).find (inx);
   if (it == (*(MSL*)m).end ())
      return 0;
   else
      return it->second;
}

void DictStringToLong::store (Key inx, Data value)
{
   // (*(MSL*)m) [inx] = value;
   (*(MSL*)m).insert (make_pair (inx, value));
}

void DictStringToLong::clear ()
{
   (*(MSL*)m).clear ();
}

DictStringToLong::DictStringToLong ()
{
   m = (void *) new MSL;
}

DictStringToLong::~DictStringToLong ()
{
   delete (MSL*) m;
}

/* DictStringToString */

#ifdef USE_STR_HASH
   typedef MAP_TYPE <string, string, StrHash> MSS;
#else
   typedef MAP_TYPE <string, string> MSS;
#endif

bool DictStringToString::contains (Key inx)
{
   return (*(MSS*)m).find (inx) != (*(MSS*)m).end ();
}

DictStringToString::Data DictStringToString::get (Key inx)
{
   MSS::iterator it = (*(MSS*)m).find (inx);
   if (it == (*(MSS*)m).end ())
      return "";
   else
      return it->second;
}

void DictStringToString::store (Key inx, Data value)
{
   // (*(MSS*)m) [inx] = value;
   (*(MSS*)m).insert (make_pair (inx, value));
}

void DictStringToString::clear ()
{
   (*(MSS*)m).clear ();
}

DictStringToString::DictStringToString ()
{
   m = (void *) new MSS;
}

DictStringToString::~DictStringToString ()
{
   delete (MSS*) m;
}

#endif // USE_OWN_DICT, USE_COMPLETE_DICT

/* ---------------------------------------------------------------------- */

#define TEST_DICTIONARY

#ifdef TEST_DICTIONARY

class C1
{
   public:
      int x, y, z;

      C1 () :
         x (0),
         y (0),
         z (0)
      { }

      C1 (const C1 & p) :
         x (p.x),
         y (p.y),
         z (p.z)
      { }
};

template <typename Key, typename Data>
void test_dict (Key a, Data b)
{
   Dictionary <Key, Data> dict;

   dict.store (a, b);
   if (dict.contains (a))
      b = dict.get (a);
}

enum color { red, blue, green, yellow };

void test_dictonaries ()
{
   test_dict <bool, string> (true, "value");
   test_dict <int, char *> (1, "value");
   test_dict <color, char *> (blue, "value");
   test_dict <string, int> ("abc", 1);

   C1 c;
   c.x = 1;
   c.y = 2;
   c.z = 3;
   // NO test_dict <C1, string> (c, "value");
   test_dict <C1 *, char *> (&c, "value");
}

#endif

/* ---------------------------------------------------------------------- */

CLOSE_NAMESPACE
