
/* envio.h */

/* Input / Output */

#ifndef __PROG_ENVIO_H__
#define __PROG_ENVIO_H__

#include "textio.h"

OPEN_NAMESPACE

string TokenKindToStr (TTokenKind kind);

/* ---------------------------------------------------------------------- */

enum TInclKind
{
   normal_incl, // used by include <file_name> and include "file_name"
   system_incl, // only for include <file_name>
   user_incl    // only for include "file_name"
};

struct TInclDir
{
   string name;
   TInclKind kind;
   TInclDir * next;

   TInclDir ();
};

struct TInclDirectories
{
   TInclDir * first_dir;
   TInclDir * last_dir;

   void InsInclDir (string name, TInclKind kind = normal_incl);
   void AddInclDir (string name, TInclKind kind = normal_incl);

   string FindInclFile (string name,
                        bool system_file = false,
                        string include_next = "",
                        bool show_incl_next = false);

   string LookupInclFile (string name,
                          string dir = "",
                          bool system_file = false,
                          string include_next = "",
                          bool show_incl_next = false);

   void CopyInclDirectoriesFrom (TInclDirectories * source);

   TInclDirectories ();
   ~ TInclDirectories ();
};

/* ---------------------------------------------------------------------- */

struct TCondSymbols;

struct TCondParam
{
   string name;
   bool multi; // true => ... parameter
   TCondParam * next;

   TCondParam ();
};

struct TCondSymbol
{
   string name;
   string value;

   bool with_params; // function like macro, defined with (possible empty) parameter list

   bool local_param; // used as local macro parameter
   string orig_value; // non-expanded parameter value

   TCondParam * first_param;
   TCondParam * last_param;

   TCondSymbol * prev;
   TCondSymbol * next;
   TCondSymbols * above;

   string FileName; // location of symbol defininiton
   int FileInx;
   int LinePos;
   int ColPos;
   int CharPos;

   void InsParam (string name, bool multi = false);
   void CopyCondParametersFrom (TCondSymbol * source);
   void DeleteParameters ();

   TCondSymbol ();
   ~ TCondSymbol ();
};

struct TCondSymbols
{
   TCondSymbol * first_symbol;
   TCondSymbol * last_symbol;

   void InsSymbol (TCondSymbol * item);

   TCondSymbol * SearchSymbol (string name);
   string RecallSymbol (string name);

   TCondSymbol * DefineNewSymbol (string name, string value);

   bool IsDefinedSymbol (string name);
   void DefineSymbolWithParams (string name, string params, string value);
   void DefineSymbol (string name, string value);
   void UndefineSymbol (string name);

   void CopyCondSymbolsFrom (TCondSymbols * source);

   TCondSymbols ();
   ~ TCondSymbols ();
};

/* ---------------------------------------------------------------------- */

struct TEnvVar
{
   string name;
   string value;
   TEnvVar * next;

   TEnvVar ();
};

struct TEnvVariables
{
   TEnvVar * first_var;
   TEnvVar * last_var;

   void InsVariable (string name, string value);
   void AddVariable (string name, string value);

   TEnvVar * SearchVariable (string name);
   string RecallVariable (string name);

   // string TranslateVariable (string name);

   void CopyEnvVariablesFrom (TEnvVariables * source);

   TEnvVariables ();
   ~ TEnvVariables ();
};

/* ---------------------------------------------------------------------- */

struct TUsedFile
{
   string name;
   TUsedFile * next;

   TUsedFile ();
};

struct TUsedFiles
{
   TUsedFile * first_file;
   TUsedFile * last_file;

   void InsUsedFile (string name);
   void AddUsedFile (string name);

   void CopyUsedFilesFrom (TUsedFiles * source);

   TUsedFiles ();
   ~ TUsedFiles ();
};

/* ---------------------------------------------------------------------- */

struct TTextEnv : public TInclDirectories,
                  public TCondSymbols,
                  public TEnvVariables,
                  public TUsedFiles
{


   // duplicate from source to empty object
   void CopyFrom (TTextEnv * source);

   string TranslatePath (string name);

   TTextEnv ();
   ~ TTextEnv ();
};

/* ---------------------------------------------------------------------- */

struct TCondLevel
{
   bool value; // true => valid branch
   bool done; // true => some valid branch already evaluated

   TCondLevel * prev;
   TCondLevel * next;

   TCondLevel ();
};

struct TCondLevels
{
   TCondLevel * first_level;
   TCondLevel * last_level;

   TCondLevel * current_level;

   int level;

   void InsCondLevel (TCondLevel * item);

   void AddCondLevel (bool value);
   void RemoveCondLevel ();
   void RemoveAllLevels ();
   bool Calculate ();

   TCondLevels ();
   ~ TCondLevels ();
};

/* ---------------------------------------------------------------------- */

enum TEnvGuard
{
   before_guard, // before first ifndef
   open_guard,   // first ifndef encountered
   inside_guard, // after first ifndef
   close_guard,  // last endif encountered
   after_guard,  // after corresponding endif
   wrong_guard   // file is not written in ifndef ... endif form
};

/* ---------------------------------------------------------------------- */

struct TEnvOptions
{
   bool SemicolonComments; // true => from semicolon to end of line is comment
   bool HashComments; // true => from hash mark to end of line is comment

   bool TextLines; // true => from reverse apostrophe ` to end of line is text literal

   bool ReportComments; // true => produce comment tokens
   bool ReportLineCont; // true => produce line continuation tokens (e.g. in macro definitions)
   bool ReportSpaces; // true => produce white space tokens
   bool ReportEol; // true => produce end of line tokens

   bool HashDirectives;   // true => directives starts with hash mark (normal situation)
   bool AtDirectives;     // true => directives can start with @ mark (not used in C language)
   bool DollarDirectives; // true => directives can start with $ mark (not used in C language)

   bool KeepEmptySections; // true => add empty lines in place of unused conditional sections
   bool KeepUnusedSections; // true => report unused conditional sections as 'UnusedDirectiveToken'

   bool RequireSymbolDefinition; // false => unknown conditional symbols are defined as zero
   bool TolerateMissingEndOfLine; // false => require end of line after directive

   bool ReportIncludes; // true => produce tokens for include directives
   bool IgnoreIncludes; // true => do not read include files, write original directive to output
   bool TolerateIncludes; // true => if include file is not found, write original directive to output

   bool IgnoreWarningDirectives; // true => warning directives are ignored
   bool IgnoreErrorDirectives; // true => error and warning directives are ignored

   bool IgnoreUnknownDirectives; // true => unknown directives are ignored

   bool IgnoreMacroExpansion; // true => do not expand macros

   bool ShowInclDirectories; // true => print information when new include directories is added
   bool ShowInclNext; // true => print information about include_next
   bool ShowIncludes; // true => print inform when include file is opened and close
   bool ShowDefinitions; // true => print information when define directives are encountered
   bool ShowMacroExpansion; // true => inform when macro is expanded
   bool ShowExpansionDetails; // true => inform when macro is expanded
   bool ShowTokens; // true => inform about each token

   TEnvOptions ();
};

/* ---------------------------------------------------------------------- */

class TSubInput : public TCharInput
{
   public:
      TCondSymbols local_symbols; // macro parameters

      TSubInput * prev;
      TSubInput * next;

      string title; // macro name, check for recursion
      string orig_include_name; // original include file name, without additional directory

      bool is_include_file;

      bool add_new_line;
      bool add_final_token;

      TSubInput * save_real_input; // file input (not macro input)

      TEnvGuard guard_state; // ifndef ... endif guard
      int guard_level;
      string guard_name;

      virtual string GetMsg () { return "included from this position"; }
      virtual string GetInfo () { return GetFileName (); }

      TSubInput ();
      virtual ~ TSubInput ();
};

class TStrSubInput : public TSubInput
{
   private:
      int inx; // position in 'value' variable

   public:
      string value; // read source text from this variable

      virtual void ReadBuf ();

      virtual string GetMsg () { return "called from this position"; }
      virtual string GetInfo () { return value; }

      TStrSubInput ();
      virtual ~ TStrSubInput ();
};

/* ---------------------------------------------------------------------- */

class TEnvInput : public TTextInput
{
   private:
      TSubInput * first_input; // original file or string input (not include or macro)
      TSubInput * real_input; // original file, string or include input (not macro input)
      TSubInput * input; // current character stream

      int sub_level;

      int line_cont; // count directive lines, only for opt.KeepEmptySections

   protected:
      TCondLevels CondTable; // nested conditional branches
      bool CondValue; // calculated result

   protected:
      TTextEnv * env; // list of defined symbols and include directories

   public:
      TEnvOptions opt;

   public:
      bool compound_separators; // multi character separators

   private:
      void GetSimpleToken ();

      void OpenSubInput (TSubInput * item);
      void CloseSubInput ();
      void CleanSubInput ();

      void OpenInclude (string file_name, string orig_name);
      void OpenMacro (TSubInput * item, bool add_stop = false);
      void OpenString (const string text, bool add_stop = false);

      void ExtraCh (); // used by multi-character separators

      TCondSymbol * LocalSearchSymbol (const string name);
      TCondSymbol * MacroSearchSymbol (TStrSubInput * macro_input, const string name);

      string ReadParameter (bool multi);
      string ExpandParameter (const string orig_value);
      void   ExpandParameters (TStrSubInput * macro_input, TCondSymbol * symbol);

      string CompoundItem (TStrSubInput * macro_input, const string name);
      void   StringSymbol (TStrSubInput * macro_input);
      void   CompoundSymbol (TStrSubInput * macro_input);

      void ExpandAgain (TCondSymbol * symbol, TStrSubInput * macro_input);
      void ExpandSymbol (TCondSymbol * symbol);
      void ResolveSymbol ();

      void SkipSpaces ();
      void CheckEndOfLine ();

      bool ThreeDots ();
      void DefineParameter (TCondSymbol * symbol);
      string DefineToText (TCondSymbol * symbol);

      bool TwoSeparators (char c1, char c2);
      void SkipTwoSeparators ();
      bool CondSymbolDefined ();

      int CondIdent (bool cont);
      int CondSimpleExpr (bool cont);
      int CondFactor (bool cont);
      int CondTerm (bool cont);
      int CondShift (bool cont);
      int CondRel (bool cont);
      int CondEq (bool cont);
      int CondAnd (bool cont);
      int CondXor (bool cont);
      int CondOr (bool cont);
      int CondLogAnd (bool cont);
      int CondLogOr (bool cont);
      int CondExpr (bool cont = true);

   protected:
      void GetExprToken ();

      void DefineDirective ();
      void UndefDirective ();
      void CondDirective (bool inv, bool only_symbol);
      void IfDirective ();
      void IfdefDirective ();
      void IfndefDirective ();
      void ElifDirective ();
      void ElseDirective ();
      void EndifDirective ();
      void IncludeDirective (bool include_next = false);
      void PragmaDirective ();
      void WarningDirective ();
      void ErrorDirective ();

   protected:
      virtual void Init ();

      void SetStartToken ();
      void SetSpaceToken (const string txt);
      void SetStringToken (const string txt);
      void SetNumberToken (const string txt);
      void SetIncludeDirectiveToken (const string txt);
      void SetUnusedDirectiveToken (const string txt);
      void SetProcessedDirectiveToken ();

      virtual bool CustomDirective (const string name) { return false; }
      virtual bool CustomPragma (const string name) { return false; }

      virtual string QueryInclude (const string file_name) { return file_name; }
      virtual void UnknownInclude (const string file_name) { }

      virtual void OpeningInclude () { }
      virtual void ClosingInclude () { }

   public:
      bool IsDefinedSymbol (string name)
      {
         return env->IsDefinedSymbol (name);
      }

      void DefineSymbol (string name, string value)
      {
         env->DefineSymbol (name, value);
      }

      void DefineSymbolWithParams (string name, string params, string value)
      {
         env->DefineSymbolWithParams (name, params, value);
      }

      void UndefineSymbol (string name)
      {
         env->UndefineSymbol (name);
      }

      TTextEnv * GetEnv () { return env; }
      void SetEnv (TTextEnv * p_env) { env = p_env; }

      void SaveOptions (TEnvOptions & state) { state = opt; }
      void RestoreOptions (TEnvOptions & state) { opt = state; }

      bool GetCondValue () { return CondValue; }

      string GetGuardSymbol ();

   public:
      virtual void GetToken ();

      void CheckEnd ();
      void ShowPositions ();

      TEnvInput ();
      virtual ~TEnvInput ();
};

CLOSE_NAMESPACE

#endif /* __PROG_ENVIO_H__ */
