
/* lookio.cc */

#include "lookio.h"
#include "str.h"

OPEN_NAMESPACE

const TInputSymbol NO_SYMBOL = 0; // !?

/********************************* QUEUE **********************************/

#if 0

/* TInputItem */

TInputItem::TInputItem () :
   symbol (NO_SYMBOL),
   kind (StartToken),
   token_txt (""),

   file_num (0),
   line_num (0),
   col_num (0),

   next (NULL),
   prev_comment (NULL),
   next_token (NULL)
{ }

void TInputItem::clear ()
{
   file_num = 0;
   line_num = 0;
   col_num = 0;

   symbol = 0;
   kind = StartToken;
   token_txt = "";

   // do not change 'next' field
   prev_comment = NULL;
   next_token = NULL;
}

#if 0
bool TInputItem::is_comment ()
{
   return kind == CommentToken ||
          kind == SpaceToken ||
          kind == EolToken ||
          kind == UnusedDirectiveToken;
}
#endif

/* TInputQueue */

void TInputQueue::insertLast (TInputItem * item)
{
   // assert (this != NULL);
   assert (item != NULL);
   assert (item->next == NULL);

   item->next = NULL;

   if (last != NULL)
      last->next = item;
   else
      first = item;

   last = item;
}

TInputItem * TInputQueue::removeFirst ()
{
   TInputItem * p = first;
   assert (p != NULL);

   first = p->next;

   if (first == NULL)
      last = NULL;

   p->next = NULL; // important
   return p;
}

void TInputQueue::insertLastFree (TInputItem * item)
{
   // assert (this != NULL);
   assert (item != NULL);
   assert (item->next == NULL);

   item->next = NULL;

   if (last_free != NULL)
      last_free->next = item;
   else
      first_free = item;

   last_free = item;
}

TInputItem * TInputQueue::removeFirstFree ()
{
   TInputItem * p = first_free;
   assert (p != NULL);

   first_free = p->next;

   if (first_free == NULL)
      last_free = NULL;

   p->next = NULL; // important
   return p;
}

void TInputQueue::removeAll ()
{
   /* DO NOT DELETE
   TInputItem * p = first;
   while (p != NULL)
   {
      TInputItem * t = p;
      p = p->next;
      delete t;
   }
   */

   first = NULL;
   last = NULL;

   TInputItem * p = first_free;
   while (p != NULL)
   {
      TInputItem * t = p;
      p = p->next;
      delete t;
   }

   first_free = NULL;
   last_free = NULL;
}

TInputQueue::~TInputQueue ()
{
   removeAll ();
}

TInputQueue::TInputQueue () :
   first (NULL),
   last (NULL),
   first_free (NULL),
   last_free (NULL)
{ }

#endif

/********************************* INPUT **********************************/

TLexerInput::TLexerInput () :
   mark_level (0),
   actual (NO_SYMBOL),

   present (NULL),
   queue (),

   comments (false)
{
}

TLexerInput::~TLexerInput ()
{
}

void TLexerInput::Init ()
{
   // original method
   TEnvInput::Init ();

   // do not read first character, already called by TTextInput::Init ()

   queue.removeAll (); // !?

   mark_level = 0;

   actual = NO_SYMBOL;
   present = NULL;
}

/* ---------------------------------------------------------------------- */

void TLexerInput::stop (string msg)
{
   Error (msg);
}

void TLexerInput::stop ()
{
   if (actual == NO_SYMBOL)
      stop ("Unexpected end of source text");
   else
      stop ("Unexpected token " + conv (actual));
}

/* ---------------------------------------------------------------------- */

#if 0
TInputItem * TLexerInput::getLocation ()
{
   if (comments)
      return present;
   else
      return NULL;
}
#endif

/**************************************************************************/

void TLexerInput::pick ()
{
   assert (present != NULL);

   actual = present->symbol;
   TokenKind = present->kind;

   TokenTxt = present->token_text;
   // TokenVal = present->token_val;
   // TokenSym = present->token_sym;

   TokenVal = ""; // !? clean
   TokenSym = ' '; // !? clean

   SetTokenFileInx (present->file_num);
   SetTokenLine (present->line_num);
   SetTokenCol (present->col_num);

   // info ("pick " + conv (actual) + " " + TokenTxt);
}

/* ---------------------------------------------------------------------- */

void TLexerInput::remember ()
{
   assert (present != NULL);

   present->symbol = actual;
   present->kind = TokenKind;

   present->token_text = TokenTxt;
   // present->token_val = TokenVal;
   // present->token_sym = TokenSym;

   TokenVal = ""; // !? clean
   TokenSym = ' '; // !? clean

   present->file_num = GetTokenFileInx ();
   present->line_num = GetTokenLine ();
   present->col_num = GetTokenCol ();

   // info ("remember " + conv (actual) + " " + TokenTxt);
}

/* ---------------------------------------------------------------------- */

#if 0
void TLexerInput::translate ()
{
   actual = NONE; // !?

   switch (TokenKind)
   {
      case IdentToken:
         actual = keyword (TokenTxt);
         if (actual == NONE)
            actual = IDENT;
         break;

      case NumberToken:
         actual = NUMBER;
         break;

      case CharToken:
         actual = CHAR_LITERAL;
         break;

      case StringToken:
         actual = STRING_LITERAL;
         break;

      case SeparatorToken:
         actual = delimiter ();
         if (actual == NONE)
             Error ("Unknown separator");
         TokenSym = 0;
         break;

      default: ;
   }

   // !?
   if (actual > STRING_LITERAL)
         TokenTxt = ""; // clear identifier variable

   // info ("read " + conv (actual) + " " + TokenTxt);
}
#endif

/* ---------------------------------------------------------------------- */

void TLexerInput::read ()
{
   TInputItem * stop = queue.last; // previous token
   TInputItem * start = NULL;
   bool done = false;

   while (! done)
   {
      GetToken ();

      if (TokenKind != CommentToken &&
          TokenKind != SpaceToken &&
          TokenKind != EolToken  &&
          TokenKind != UnusedDirectiveToken )
      {
         done = true;
      }

      if (done || comments)
      {
         // add new item into queue
         if (queue.first_free != NULL)
         {
            present = queue.removeFirstFree ();
            present->clear ();
         }
         else
         {
            present = new TInputItem;
         }
         queue.insertLast (present);
      }

      actual = NO_SYMBOL; // !?

      // translate TokenKind to TInputSymbol
      if (done)
         translate ();

      // store token
      if (done || comments)
         remember ();

      if (start == NULL)
         start = present;
   }

   // pointer to previous comment
   present->prev_comment = start;

   // link previous token
   if (stop != NULL)
      stop->next_token = present;
}

/* ---------------------------------------------------------------------- */

void TLexerInput::next ()
// read next token
{
   // info ("nxt");

   if (present != NULL)
   {
      // !? !!
      #if 0
      if (! comments && mark_level == 0) // no locations, notes and not in mark/rewind
      {
         // move present item to free list
         assert (present == queue.first);
         TInputItem * item = queue.removeFirst ();
         item->prev_comment = NULL;
         item->next_token = NULL;
         queue.insertLastFree (item);

         // set actual item
         present = queue.first;
      }
      else
      #endif
      {
         // move to actual item
         present = present->next_token;
      }
   }

   if (present == NULL)
   {
      // read next token
      read ();
   }
   else
   {
      // use item from queue
      pick ();
   }

   // info ("next " + conv (actual) + " " + TokenTxt);
}

/**************************************************************************/

TInputItem * TLexerInput::mark ()
// remember current position
{
   mark_level ++;
   return present;
}

void TLexerInput::rewind (TInputItem * pos)
// return to previous position
{
   mark_level --;
   present = pos;
   pick ();
}

void TLexerInput::skip_rewind ()
{
   mark_level --;
   // !?
}

/**************************************************************************/

void TLexerInput::match (TInputSymbol s)
// compare token and read next token
{
   if (actual != s)
   {
      stop (QuoteStr (conv (s)) + " expected");
   }

   if (s != NO_SYMBOL)
      next ();
}

/* ---------------------------------------------------------------------- */

TInputSymbol TLexerInput::LA (int i)
{
   TInputSymbol result;

   if (i == 1)
   {
      result = actual;
   }
   else
   {
      // !? not efficient
      TInputItem * inx = mark ();
      for (int k = 1; k <= i-1; k++)
         next ();
      result = actual;
      rewind (inx);
   }

   // info ("LA (" + IntToStr (i) + ")  = " + conv (result));
   return result;
}

/**************************************************************************/

#if 0
string TLexerInput::recall_identifier ()
{
   if (actual != IDENT)
      Error ("identifier expected");
   string s = TokenTxt;
   next ();
   return s;
}

string TLexerInput::recall_numeric_literal ()
{
   if (actual != NUMBER)
      Error ("Numeric literal expected");
   string s = TokenTxt;
   next ();
   return s;
}

string TLexerInput::recall_char_literal ()
{
   if (actual != CHAR_LITERAL)
      Error ("Character literal expected");
   string s = TokenTxt; // !?
   next ();
   return s;
}

string TLexerInput::recall_string_literal ()
{
   if (actual != STRING_LITERAL)
      Error ("String literal expected");
   string s = TokenTxt; // !?
   next ();
   return s;
}
#endif

/********************************* OUTPUT *********************************/

/* ---------------------------------------------------------------------- */

CLOSE_NAMESPACE
